from contextlib import contextmanager
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base


class DBInit(object):
    def __init__(self):
        self.Model = None
        self.session_factory = None

    def init_engine(self, url=None, **kwargs):
        if kwargs:
            self.engine = create_engine(url, **kwargs)
        else:
            self.engine = create_engine(url)

        self.session_factory = sessionmaker(bind=self.engine)
        self.Model = declarative_base()
        self.Model.metadata.create_all(self.engine)

    @contextmanager
    def session_maker(self):
        if not self.session_factory:
            raise Exception("db initial error")

        try:
            session = self.session_factory()
            print("<<<<<<<<<<<<<<< start")
            yield session
            session.commit()
            print("<<<<<<<<<<<<<<< commit <<<<<<<<")
        except:
            session.rollback()
            print("<<<<<<<<<<<<<<< rollback <<<<<<<<")
            raise
        finally:
            session.close()
            print("<<<<<<<<<<<<<<< close <<<<<<<<")


from ext.es.main import ESTool

ES = None


def create_es(config):
    _hosts = config.ELASTICSEARCH_HOST
    if not _hosts:
        return

    _instance = ESTool()
    _instance.init_app(_hosts)

    return _instance



from ext.db import create_db
from ext.es import create_es
from ext.redis import create_redis

EXT_CONTEXT = {

}


def create_ext(config):
    if not config:
        return

    global EXT_CONTEXT

    if not EXT_CONTEXT.get("db"):
        _db = create_db(config)
        EXT_CONTEXT["db"] = _db

    if not EXT_CONTEXT.get("redis"):
        _redis = create_redis(config)
        EXT_CONTEXT["redis"] = _redis

    if not EXT_CONTEXT.get("es"):
        _es = create_es(config)
        EXT_CONTEXT["es"] = _es



import redis

REDIS = None


def create_redis(config):
    _redis_config = config.REDIS
    if not _redis_config:
        return

    _host = _redis_config.get("HOST")
    _port = _redis_config.get("PORT")

    _instance = redis.Redis(host=_host, port=_port, decode_responses=True)
    return _instance



from ext.db.main import DBInit

DB = None


def create_db(config):
    _db_url = config.SQLALCHEMY_DATABASE_URI
    if not _db_url:
        return

    _instance = DBInit()
    _instance.init_engine(url=_db_url)
    return _instance



# -*- coding: utf8 -*-

from elasticsearch_dsl import Search
from elasticsearch import Elasticsearch


class ESTool(object):
    def __init__(self, hosts=None):
        self.conn = None
        if hosts:
            self.conn = Elasticsearch([hosts], max_retries=0)

    def init_app(self, hosts):
        if hosts:
            self.conn = Elasticsearch([hosts], max_retries=0)

    def search(self, index, fields, mapping, **kwargs):
        """构建查询"""
        res = Search(using=self.conn, index=index).source(fields)
        for key in kwargs.keys():
            res = res.filter("term", **{mapping[key]: kwargs[key]})
        res = res.scan()
        return [hit.to_dict() for hit in res]

    # 检测index是否存在
    def index_exists(self, index):
        return self.conn.indices.exists(index=index)

    def add_index(self, index, body=None):
        self.conn.indices.create(index=index, ignore=400, body=body)

    def set_settings(self, index, settings):
        self.conn.indices.get_settings(
            index=index, doc_type=index, body=settings)

    def set_mapping(self, index, mapping):
        self.conn.indices.put_mapping(
            index=index, doc_type=index, body=mapping)

    def add_data(self, index, data):
        self.conn.index(index=index, doc_type=index, body=data)

    def del_index(self, index):
        self.conn.indices.delete(index=index)


if __name__ == "__main__":
    # result = ESTool(hosts="10.249.151.23:9200").search(
    #     index="noah_alarm_yunsuo_20200209",
    #     fields=["unique_id"],
    #     mapping={"key": "unique_id"},
    #     key="1577195407132909334",
    # )
    # print(result)
    pass



import datetime

from sqlalchemy.orm import class_mapper
from ext import EXT_CONTEXT


def object_to_dict(obj):
    columns = [column.key for column in class_mapper(obj.__class__).columns]
    get_key_value = lambda c: (c, getattr(obj, c).isoformat()) if isinstance(getattr(obj, c), datetime) else (
        c, getattr(obj, c))
    return dict(map(get_key_value, columns))


def query2dict(model_list):
    if isinstance(model_list, list):  # 如果传入的参数是一个list类型的，说明是使用的all()的方式查询的
        if isinstance(model_list[0], EXT_CONTEXT["db"].Model):  # 这种方式是获得的整个对象  相当于 select * from table
            lst = []
            for model in model_list:
                dic = {}
                for col in model.__table__.columns:
                    dic[col.name] = getattr(model, col.name)
                lst.append(dic)
            return lst
        else:  # 这种方式获得了数据库中的个别字段  相当于select id,name from table
            lst = []
            for result in model_list:  # 当以这种方式返回的时候，result中会有一个keys()的属性
                lst.append([dict(zip(result.keys, r)) for r in result])
            return lst
    else:  # 不是list,说明是用的get() 或者 first()查询的，得到的结果是一个对象
        if isinstance(model_list, EXT_CONTEXT["db"].Model):  # 这种方式是获得的整个对象  相当于 select * from table limit=1
            dic = {}
            for col in model_list.__table__.columns:
                dic[col.name] = getattr(model_list, col.name)
            return dic
        else:  # 这种方式获得了数据库中的个别字段  相当于select id,name from table limit = 1
            return dict(zip(model_list.keys(), model_list))


#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages

setup(
    name="testext",
    version="1.0.1",
    keywords=("test", "ext"),
    description="ext sdk",
    long_description="ext sdk for python",
    license="MIT Licence",

    url="",
    author="skyeye",
    author_email="skyeye@gmail.com",

    packages=find_packages(exclude=['dist']),
    include_package_data=True,
    platforms="any",
    install_requires=[
        'psycopg2==2.8.6',
        'sqlalchemy==1.3.22',
        'elasticsearch-dsl==7.3.0',
        'elasticsearch==7.9.1',
        'redis==3.5.3'
    ],

    scripts=[],
    # entry_points={
    #     'console_scripts': [
    #         'test = test.help:main'
    #     ]
    # }
)



import sys, types


def _get_mod(module_path):
    try:
        aMod = sys.modules[module_path]
        if not isinstance(aMod, types.ModuleType):
            raise KeyError
    except KeyError:
        # The last [''] is very important!
        aMod = __import__(module_path, globals(), locals(), [''])
        sys.modules[module_path] = aMod
    return aMod


def _get_func(full_func_name):
    """Retrieve a function object from a full dotted-package name."""
    # Parse out the path, module, and function
    lastDot = full_func_name.rfind(u".")
    funcName = full_func_name[lastDot + 1:]
    modPath = full_func_name[:lastDot]
    aMod = _get_mod(modPath)
    aFunc = getattr(aMod, funcName)
    # Assert that the function is a *callable* attribute.
    assert callable(aFunc), u"%s is not callable." % full_func_name
    # Return a reference to the function itself,
    # not the results of the function.
    return aFunc


def _get_Class(full_class_name, parent_class=None):
    """Load a module and retrieve a class (NOT an instance).
  If the parentClass is supplied, className must be of parentClass
  or a subclass of parentClass (or None is returned).
  """
    aClass = _get_func(full_class_name)
    # Assert that the class is a subclass of parentClass.
    if parent_class is not None:
        if not issubclass(aClass, parent_class):
            raise TypeError(u"%s is not a subclass of %s" %
                            (full_class_name, parent_class))
    # Return a reference to the class itself, not an instantiated object.
    return aClass


# def applyFuc(obj,strFunc,arrArgs):
#   objFunc = getattr(obj, strFunc)
#   # return apply(objFunc,arrArgs)
#   return apply(objFunc,arrArgs)


def _get_object(full_class_name):
    clazz = _get_Class(full_class_name)
    return clazz()
